/*
This library demonstrates a very simple approach to drawing elevated terrain.

The good part: each icon only has to have a few icon states: the nameless base, plus
shady, bright, bs, and sb (see desert.dmi or lawn.dmi for examples). This is essentially the same
trick that GUI interfaces (e.g., Windows) use to create the appearance of buttons that are slightly
raised or lowered: the upper and left edges are lighter than the main color, and the lower and right
edges are darker. In everyday life, most of the lighting we see comes from above, so our minds
automatically try to interpret this shading trick in a way that makes sense.

The library supports setting turf elevations from icon states with numeric names. Again, see the dmi
files, and see the map as well -- if for no other reason than to admire the nice clear visual guide
on the teeny overview map, which is created by diagonal lines of varying thickness. If you're going
to write your own random terrain generator (or random elevation generator for pre-laid turfs),
you don't need to worry about creating those numeric states; just set all the elevations before you
call DrawElevations().

Now the bad part: because of the simplistic nature of the shading, you can get some pretty quirky
results if you try to draw fine detail. It's best to lay down turfs in 3x3 blocks of the same
elevation. This makes sense when you see how it works -- a 3x3 block will give you a flat central
"plateau" turf, with the eight surrounding turfs shaded.

Given its many limitations, it's probably more of a curiosity than a truly useful library,
but it's kind of fun anyway.
*/


//Call DrawElevations when you want to render the turfs' elevations on the map.
//TURFS_BEFORE_SLEEP is defined in the .dme file. It's intended to help prevent the world from
//freezing up. If that's not a concern for you, just set TURFS_BEFORE_SLEEP to a very high number.
proc/DrawElevations()
	var/turf/elevated/myTurf
	var/i = 0

	for(myTurf in world)
		myTurf.DrawElevations()
		if(!(++i % TURFS_BEFORE_SLEEP))
			sleep


turf/elevated
	var/elevation = 0


	New()
		. = ..()
		elevation = text2num(icon_state)
		icon_state = ""


	proc/DrawElevations()
		var/turf/elevated/myStep
		var/myDir
		var/borders = 0

		borders = 0
		for(myDir in list(1, 2, 4, 8))
			myStep = get_step(src, myDir)

			if((!istype(myStep)) || (istype(myStep) && (myStep.elevation >= elevation)))
				borders += myDir

		switch(borders)
			if(5, 10)
				icon_state = "bs"
			if(9, 11, 13)
				icon_state = "shady"
			if(6, 7, 14)
				icon_state = "bright"
			if(15)

				borders = 0
				for(myDir in list(5, 6, 9, 10))
					myStep = get_step(src, myDir)
					if(myStep && (myStep.elevation < elevation))
						borders += myDir

				switch(borders)
					if(5, 10)
						icon_state = "sb"
					if(6)
						icon_state = "shady"
					if(9)
						icon_state = "bright"
